"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react")); // eslint-disable-line
const prop_types_1 = __importDefault(require("prop-types"));
const react_native_1 = require("react-native");
const FloatingActionItem_1 = __importDefault(require("./FloatingActionItem"));
const AddIcon_1 = __importDefault(require("./AddIcon"));
const platform_1 = require("./utils/platform");
const touchable_1 = require("./utils/touchable");
const SceneUtils_1 = __importDefault(require("../../utils/SceneUtils"));
const DEVICE_WIDTH = react_native_1.Dimensions.get("window").width;
const DEFAULT_SHADOW_PROPS = {
    shadowOpacity: 0.35,
    shadowOffset: {
        width: 0,
        height: 5
    },
    shadowColor: "#000000",
    shadowRadius: 3
};
class FloatingAction extends react_1.default.Component {
    constructor(props) {
        super(props);
        this.onKeyboardShow = (e) => {
            const { buttonSize, actionsPaddingTopBottom } = this.props;
            const { height } = e.endCoordinates;
            react_native_1.Animated.parallel([
                react_native_1.Animated.spring(this.actionsBottomAnimation, {
                    bounciness: 0,
                    toValue: buttonSize +
                        this.distanceToVerticalEdge +
                        actionsPaddingTopBottom +
                        height -
                        ((0, platform_1.isIphoneX)() ? 40 : 0),
                    // duration: 250,
                    useNativeDriver: false
                }),
                react_native_1.Animated.spring(this.mainBottomAnimation, {
                    bounciness: 0,
                    toValue: this.distanceToVerticalEdge + height - ((0, platform_1.isIphoneX)() ? 40 : 0),
                    // duration: 250,
                    useNativeDriver: false
                })
            ]).start();
        };
        this.onKeyboardHideHide = () => {
            const { buttonSize, actionsPaddingTopBottom } = this.props;
            react_native_1.Animated.parallel([
                react_native_1.Animated.spring(this.actionsBottomAnimation, {
                    bounciness: 0,
                    toValue: buttonSize + this.distanceToVerticalEdge + actionsPaddingTopBottom,
                    // duration: 250,
                    useNativeDriver: false
                }),
                react_native_1.Animated.spring(this.mainBottomAnimation, {
                    bounciness: 0,
                    toValue: this.distanceToVerticalEdge,
                    // duration: 250,
                    useNativeDriver: false
                })
            ]).start();
        };
        this.getShadow = () => {
            const { shadow } = this.props;
            return Object.assign(Object.assign({}, DEFAULT_SHADOW_PROPS), shadow);
        };
        this.getIcon = () => {
            const { iconWidth, iconHeight, } = this.props;
            return react_1.default.createElement(AddIcon_1.default, { width: iconWidth, height: iconHeight, backgroundColor: this.state.active ? SceneUtils_1.default.primary_color : 'black' });
        };
        this.reset = () => {
            const { animated, onClose } = this.props;
            if (animated) {
                react_native_1.Animated.spring(this.animation, { toValue: 0, useNativeDriver: false }).start();
                react_native_1.Animated.spring(this.actionsAnimation, { toValue: 0, useNativeDriver: false }).start();
            }
            this.updateState({
                active: false
            }, () => {
                if (onClose) {
                    onClose();
                }
            });
        };
        this.animateButton = () => {
            const { overrideWithAction, actions, floatingIcon, animated, dismissKeyboardOnPress, onPressMain, onOpen } = this.props;
            const { active } = this.state;
            if (dismissKeyboardOnPress) {
                react_native_1.Keyboard.dismiss();
            }
            if (overrideWithAction) {
                this.handlePressItem(actions[0].name);
                return;
            }
            if (onPressMain) {
                onPressMain(!active);
            }
            if (!active) {
                if (!floatingIcon) {
                    if (animated) {
                        react_native_1.Animated.spring(this.animation, { toValue: 1, useNativeDriver: false }).start();
                    }
                }
                if (animated) {
                    react_native_1.Animated.spring(this.actionsAnimation, { toValue: 1, useNativeDriver: false }).start();
                    // only execute it for the background to prevent extra calls
                    react_native_1.LayoutAnimation.configureNext({
                        duration: 180,
                        create: {
                            type: react_native_1.LayoutAnimation.Types.easeInEaseOut,
                            property: react_native_1.LayoutAnimation.Properties.opacity
                        }
                    });
                }
                this.updateState({
                    active: true
                }, () => {
                    if (onOpen) {
                        onOpen();
                    }
                });
            }
            else {
                this.reset();
            }
        };
        this.updateState = (nextState, callback) => {
            const { onStateChange } = this.props;
            const { active } = this.state;
            this.setState(nextState, () => {
                if (callback) {
                    callback();
                }
                if (onStateChange) {
                    onStateChange({
                        isActive: active
                    });
                }
            });
        };
        this.handlePressBackdrop = () => {
            const { onPressBackdrop } = this.props;
            if (onPressBackdrop) {
                onPressBackdrop();
            }
            this.reset();
        };
        this.handlePressItem = (itemName) => {
            const { onPressItem } = this.props;
            if (onPressItem) {
                onPressItem(itemName);
            }
            //if((itemName !== 'add_to_bag'  || !bodyDetailsSaved) && itemName !== 'add_to_wishlist') 
            this.reset();
        };
        this.state = {
            active: false
        };
        this.mainBottomAnimation = new react_native_1.Animated.Value(this.distanceToVerticalEdge + props.mainVerticalDistance);
        this.actionsBottomAnimation = new react_native_1.Animated.Value(props.buttonSize +
            this.distanceToVerticalEdge +
            props.actionsPaddingTopBottom +
            props.mainVerticalDistance);
        this.animation = new react_native_1.Animated.Value(0);
        this.actionsAnimation = new react_native_1.Animated.Value(0);
        this.visibleAnimation = new react_native_1.Animated.Value(props.visible ? 0 : 1);
        /*
         * this animation will fix an error on ReactNative (Android) where
         * interpolations with 0 and 1 don't work as expected.
         */
        this.fadeAnimation = new react_native_1.Animated.Value(props.visible ? 1 : 0);
    }
    componentDidMount() {
        const { openOnMount, listenKeyboard } = this.props;
        if (openOnMount) {
            this.animateButton();
        }
        if (listenKeyboard) {
            const showEvent = react_native_1.Platform.OS === "ios" ? "keyboardWillShow" : "keyboardDidShow";
            const hideEvent = react_native_1.Platform.OS === "ios" ? "keyboardWillHide" : "keyboardDidHide";
            this.keyboardWillShowListener = react_native_1.Keyboard.addListener(showEvent, this.onKeyboardShow);
            this.keyboardWillHideListener = react_native_1.Keyboard.addListener(hideEvent, this.onKeyboardHideHide);
        }
    }
    componentDidUpdate(prevProps) {
        const { visible } = this.props;
        if (prevProps.visible !== visible) {
            if (visible) {
                react_native_1.Animated.parallel([
                    react_native_1.Animated.spring(this.visibleAnimation, { toValue: 0, useNativeDriver: false }),
                    react_native_1.Animated.spring(this.fadeAnimation, { toValue: 1, useNativeDriver: false })
                ]).start();
            }
            if (!visible) {
                react_native_1.Animated.parallel([
                    react_native_1.Animated.spring(this.visibleAnimation, { toValue: 1, useNativeDriver: false }),
                    react_native_1.Animated.spring(this.fadeAnimation, { toValue: 0, useNativeDriver: false })
                ]).start();
            }
        }
    }
    componentWillUnmount() {
        const { listenKeyboard } = this.props;
        if (listenKeyboard) {
            this.keyboardWillShowListener.remove();
            this.keyboardWillHideListener.remove();
        }
    }
    get distanceToHorizontalEdge() {
        const { distanceToEdge } = this.props;
        return typeof distanceToEdge === 'number'
            ? distanceToEdge
            : distanceToEdge.horizontal;
    }
    get distanceToVerticalEdge() {
        const { distanceToEdge } = this.props;
        return typeof distanceToEdge === 'number'
            ? distanceToEdge
            : distanceToEdge.vertical;
    }
    renderMainButton() {
        const { 
        // @deprecated in favor of "color"
        buttonColor, // eslint-disable-line
        buttonSize, color, position, overrideWithAction, animated } = this.props;
        const { active } = this.state;
        if (buttonColor) {
            console.warn('FloatingAction: "buttonColor" property was deprecated. Please use "color"');
        }
        let animatedVisibleView;
        let animatedViewStyle;
        if (animated) {
            animatedVisibleView = {
                opacity: this.fadeAnimation,
                transform: [
                    {
                        rotate: this.visibleAnimation.interpolate({
                            inputRange: [0, 1],
                            outputRange: ["0deg", "90deg"]
                        })
                    },
                    {
                        scale: this.visibleAnimation.interpolate({
                            inputRange: [0, 1],
                            outputRange: [1, 0]
                        })
                    }
                ]
            };
            animatedViewStyle = {
                transform: [
                    {
                        rotate: this.animation.interpolate({
                            inputRange: [0, 1],
                            outputRange: ["0deg", "45deg"]
                        })
                    }
                ]
            };
            if (overrideWithAction) {
                animatedViewStyle = {};
            }
        }
        else if (active) {
            animatedVisibleView = {};
            animatedViewStyle = {
                transform: [
                    {
                        rotate: "45deg"
                    }
                ]
            };
        }
        else {
            animatedVisibleView = {};
            animatedViewStyle = {
                transform: [
                    {
                        rotate: "0deg"
                    }
                ]
            };
        }
        const Touchable = (0, touchable_1.getTouchableComponent)();
        const propStyles = {
            backgroundColor: '#FFF',
            bottom: this.mainBottomAnimation // I need to imporove this to run on native thread and not on JS thread
        };
        if (["left", "right"].indexOf(position) > -1) {
            propStyles[position] = this.distanceToHorizontalEdge;
        }
        const sizeStyle = {
            width: buttonSize,
            height: buttonSize,
            borderRadius: buttonSize / 2
        };
        return (react_native_1.Platform.OS === 'ios' ?
            react_1.default.createElement(react_native_1.Animated.View, { style: [
                    styles.buttonContainer,
                    sizeStyle,
                    styles[`${position}Button`],
                    propStyles,
                    animatedVisibleView,
                    { backgroundColor: 'rgb(255,255,255)', top: react_native_1.Platform.OS === 'android' ? 10 : 0 }
                ], accessible: true, accessibilityLabel: "Floating Action Button" },
                react_1.default.createElement(Touchable, Object.assign({}, (0, touchable_1.getRippleProps)('#FFF'), { style: [styles.button, sizeStyle], activeOpacity: 0.85, onPress: this.animateButton }),
                    react_1.default.createElement(react_native_1.Animated.View, { style: [styles.buttonTextContainer, sizeStyle] }, this.getIcon())))
            :
                react_1.default.createElement(react_native_1.TouchableOpacity, Object.assign({}, (0, touchable_1.getRippleProps)('#FFF'), { style: [styles.button, sizeStyle, { height: 48, width: 48, borderRadius: 24, backgroundColor: '#FFF' }], activeOpacity: 0.85, onPress: this.animateButton }),
                    react_1.default.createElement(react_native_1.Animated.View, { style: [styles.buttonTextContainer, sizeStyle, { height: 48, width: 48, borderRadius: 24, backgroundColor: '#FFF' }] }, this.getIcon())));
    }
    renderActions() {
        const { actions, position, overrideWithAction, animated } = this.props;
        const { active } = this.state;
        if (!actions || actions.length === 0) {
            return undefined;
        }
        if (overrideWithAction) {
            return null;
        }
        let animatedActionsStyle;
        if (animated) {
            animatedActionsStyle = {
                opacity: this.actionsAnimation.interpolate({
                    inputRange: [0, 1],
                    outputRange: [0, 1]
                })
            };
        }
        else {
            animatedActionsStyle = { opacity: active ? 1 : 0 };
        }
        const actionsStyles = [
            styles.actions,
            styles[`${position}Actions`],
            animatedActionsStyle,
            {
                bottom: this.actionsBottomAnimation
            }
        ];
        if (active) {
            actionsStyles.push(styles[`${position}ActionsVisible`]);
        }
        const sortedActions = actions.sort((a, b) => a.position - b.position);
        return (react_1.default.createElement(react_native_1.Animated.View, { style: actionsStyles, pointerEvents: "box-none" }, sortedActions.map((action, index) => {
            const textColor = action.textColor || action.actionsTextColor;
            const textBackground = action.textBackground || action.actionsTextBackground;
            return (react_1.default.createElement(FloatingActionItem_1.default, Object.assign({ key: action.name, textColor: textColor, textBackground: textBackground }, action, { cartCount: this.props.cartCount, position: position, active: active, onPress: this.handlePressItem, animated: animated, index: index })));
        })));
    }
    renderTappableBackground() {
        const { overlayColor } = this.props;
        // TouchableOpacity don't require a child
        return (react_1.default.createElement(react_native_1.TouchableOpacity, { activeOpacity: 1, style: [styles.overlay], onPress: this.handlePressBackdrop }));
    }
    render() {
        const { active } = this.state;
        const { showBackground } = this.props;
        return (react_1.default.createElement(react_native_1.Animated.View, { pointerEvents: "box-none", style: [styles.overlay] },
            active && showBackground && this.renderTappableBackground(),
            this.renderActions(),
            this.renderMainButton()));
    }
}
FloatingAction.propTypes = {
    actions: prop_types_1.default.arrayOf(prop_types_1.default.shape({
        color: prop_types_1.default.string,
        icon: prop_types_1.default.any,
        name: prop_types_1.default.string.isRequired,
        buttonSize: prop_types_1.default.number,
        text: prop_types_1.default.string,
        textBackground: prop_types_1.default.string,
        textColor: prop_types_1.default.string,
        component: prop_types_1.default.func,
        animated: prop_types_1.default.bool
    })),
    animated: prop_types_1.default.bool,
    color: prop_types_1.default.string,
    distanceToEdge: prop_types_1.default.oneOfType([
        prop_types_1.default.number,
        prop_types_1.default.shape({
            vertical: prop_types_1.default.number,
            horizontal: prop_types_1.default.number
        })
    ]),
    mainVerticalDistance: prop_types_1.default.number,
    visible: prop_types_1.default.bool,
    overlayColor: prop_types_1.default.string,
    position: prop_types_1.default.oneOf(["right", "left", "center"]),
    overrideWithAction: prop_types_1.default.bool,
    floatingIcon: prop_types_1.default.any,
    showBackground: prop_types_1.default.bool,
    openOnMount: prop_types_1.default.bool,
    actionsPaddingTopBottom: prop_types_1.default.number,
    buttonSize: prop_types_1.default.number,
    iconHeight: prop_types_1.default.number,
    iconWidth: prop_types_1.default.number,
    listenKeyboard: prop_types_1.default.bool,
    dismissKeyboardOnPress: prop_types_1.default.bool,
    shadow: prop_types_1.default.shape({
        shadowOpacity: prop_types_1.default.number,
        shadowOffset: prop_types_1.default.shape({
            width: prop_types_1.default.number,
            height: prop_types_1.default.number
        }),
        shadowColor: prop_types_1.default.string,
        shadowRadius: prop_types_1.default.number
    }),
    onPressItem: prop_types_1.default.func,
    onPressMain: prop_types_1.default.func,
    onClose: prop_types_1.default.func,
    onOpen: prop_types_1.default.func,
    onPressBackdrop: prop_types_1.default.func,
    onStateChange: prop_types_1.default.func
};
FloatingAction.defaultProps = {
    dismissKeyboardOnPress: false,
    listenKeyboard: false,
    actionsPaddingTopBottom: 8,
    overrideWithAction: false,
    visible: true,
    color: SceneUtils_1.default.primary_color,
    overlayColor: "rgba(68, 68, 68, 0.6)",
    position: "right",
    distanceToEdge: 30,
    openOnMount: false,
    showBackground: true,
    buttonSize: 48,
    iconHeight: 14,
    iconWidth: 14,
    iconColor: '#fff',
    mainVerticalDistance: 0,
    animated: true,
    shadow: {}
};
const styles = react_native_1.StyleSheet.create({
    actions: {
        position: "absolute",
        bottom: 85,
        zIndex: 10
    },
    rightActions: {
        alignItems: "flex-end",
        right: -1000 // this magic number will make always disspear the text from screen
    },
    leftActions: {
        alignItems: "flex-start",
        left: -1000 // this magic number will make always disspear the text from screen
    },
    centerActions: {
        left: -1000
    },
    rightActionsVisible: {
        right: 0
    },
    leftActionsVisible: {
        left: 0
    },
    centerActionsVisible: {
        left: DEVICE_WIDTH / 2 - 30
    },
    overlay: {
        position: "absolute",
        bottom: 0,
        right: 0,
        // right: -15,
        top: 0,
        zIndex: 0
    },
    buttonContainer: {
        overflow: react_native_1.Platform.OS === "ios" ? "visible" : "hidden",
        zIndex: 2,
        alignItems: "center",
        justifyContent: "center",
        position: "absolute"
    },
    button: {
        zIndex: 3,
        alignItems: "center",
        justifyContent: "center"
    },
    rightButton: {},
    leftButton: {},
    centerButton: {
        left: DEVICE_WIDTH / 2 - 28
    },
    buttonTextContainer: {
        flex: 1,
        justifyContent: "center",
        alignItems: "center"
    }
});
exports.default = FloatingAction;
//# sourceMappingURL=FloatingAction.js.map