"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const prop_types_1 = __importDefault(require("prop-types"));
const react_native_1 = require("react-native");
const touchable_1 = require("./utils/touchable");
const SceneUtils_1 = __importDefault(require("../../utils/SceneUtils"));
const react_native_gesture_handler_1 = require("react-native-gesture-handler");
class FloatingActionItem extends react_1.default.Component {
    constructor(props) {
        super(props);
        this.handleOnPress = () => {
            const { name, onPress } = this.props;
            onPress(name);
        };
        this.animation = new react_native_1.Animated.Value(0);
    }
    componentDidUpdate(prevProps) {
        const { active, animated } = this.props;
        if (prevProps.active !== active && animated) {
            react_native_1.Animated.spring(this.animation, {
                toValue: active ? 1 : 0,
                useNativeDriver: false
            }).start();
        }
    }
    get distanceToHorizontalEdge() {
        const { distanceToEdge } = this.props;
        return typeof distanceToEdge === 'number'
            ? distanceToEdge
            : distanceToEdge.horizontal;
    }
    get distanceToVerticalEdge() {
        const { distanceToEdge } = this.props;
        return typeof distanceToEdge === 'number'
            ? distanceToEdge
            : distanceToEdge.vertical;
    }
    renderText() {
        const { text, position, textColor, textStyle, textProps, textContainerStyle, } = this.props;
        if (text) {
            return (react_1.default.createElement(react_native_1.View, { key: "text", style: [
                    styles.textContainer,
                    styles[`${position}TextContainer`],
                    textContainerStyle
                ] },
                react_1.default.createElement(react_native_1.Text, Object.assign({ style: [
                        styles.text,
                        {
                            color: textColor
                        },
                        textStyle
                    ] }, textProps), text),
                this.props.text === 'GO TO BAG' &&
                    react_1.default.createElement(react_native_1.View, { style: styles.countContainer },
                        react_1.default.createElement(react_native_1.Text, { style: styles.countText }, this.props.cartCount))));
        }
        return null;
    }
    renderButton() {
        const { buttonSize, icon, color, shadow, tintColor } = this.props;
        if (icon && icon.uri) {
            iconStyle = styles.iconLogo;
        }
        else {
            iconStyle = styles.icon;
        }
        const propStyles = {
            tintColor: tintColor,
            backgroundColor: color,
            width: buttonSize,
            height: buttonSize,
            borderRadius: buttonSize / 2
        };
        return (react_1.default.createElement(react_native_1.View, { key: "button", style: styles.button },
            react_1.default.createElement(react_native_1.Image, { style: styles.icon, source: icon })));
    }
    render() {
        const { position, render, margin, name, animated } = this.props;
        const Touchable = (0, touchable_1.getTouchableComponent)(false);
        let animatedActionContainerStyle;
        if (animated) {
            animatedActionContainerStyle = {
                marginBottom: this.animation.interpolate({
                    inputRange: [0, 1],
                    outputRange: [1, 3]
                })
            };
        }
        else {
            animatedActionContainerStyle = { marginBottom: 6 };
        }
        const components = [];
        components.push(this.renderButton());
        components.push(this.renderText());
        return (react_1.default.createElement(Touchable, { activeOpacity: 0.4, style: styles.container },
            react_1.default.createElement(react_native_gesture_handler_1.TouchableOpacity, { onPress: this.handleOnPress },
                react_1.default.createElement(react_native_1.Animated.View, { style: [
                        styles.actionContainer,
                        animatedActionContainerStyle,
                        styles[`${position}ActionContainer`],
                        {
                            borderTopLeftRadius: this.props.index === 0 ? 6 : 0,
                            borderTopRightRadius: this.props.index === 0 ? 6 : 0,
                            borderBottomLeftRadius: this.props.index === 1 ? 6 : 0,
                            borderBottomRightRadius: this.props.index === 1 ? 6 : 0
                        }
                    ] }, components))));
    }
}
FloatingActionItem.propTypes = {
    tintColor: prop_types_1.default.string,
    color: prop_types_1.default.string,
    icon: prop_types_1.default.any,
    name: prop_types_1.default.string.isRequired,
    buttonSize: prop_types_1.default.number,
    textContainerStyle: prop_types_1.default.object,
    text: prop_types_1.default.string,
    textStyle: prop_types_1.default.object,
    textProps: prop_types_1.default.object,
    textBackground: prop_types_1.default.string,
    textColor: prop_types_1.default.string,
    shadow: prop_types_1.default.shape({
        shadowOpacity: prop_types_1.default.number,
        shadowOffset: prop_types_1.default.shape({
            width: prop_types_1.default.number,
            height: prop_types_1.default.number
        }),
        shadowColor: prop_types_1.default.string,
        shadowRadius: prop_types_1.default.number
    }),
    // not modified by user
    position: prop_types_1.default.oneOf(["left", "right", "center"]),
    active: prop_types_1.default.bool,
    distanceToEdge: prop_types_1.default.oneOfType([
        prop_types_1.default.number,
        prop_types_1.default.shape({
            vertical: prop_types_1.default.number,
            horizontal: prop_types_1.default.number
        })
    ]),
    onPress: prop_types_1.default.func,
    render: prop_types_1.default.func,
    margin: prop_types_1.default.number,
    animated: prop_types_1.default.bool
};
FloatingActionItem.defaultProps = {
    tintColor: '#fff',
    color: SceneUtils_1.default.primary_color,
    distanceToEdge: 30,
    buttonSize: 40,
    textColor: SceneUtils_1.default.text_color,
    textBackground: "#ffffff",
    margin: 8,
    shadow: {
        shadowOpacity: 0.35,
        shadowOffset: {
            width: 0,
            height: 5
        },
        shadowColor: "#000000",
        shadowRadius: 3
    }
};
const styles = react_native_1.StyleSheet.create({
    container: {
        flex: 1,
        flexDirection: "column"
    },
    actionContainer: {
        flex: 1,
        flexDirection: "row",
        alignItems: 'center',
        paddingLeft: 0,
        paddingRight: 0,
        backgroundColor: '#FFF',
        width: 220,
        marginRight: 30,
        height: 48
    },
    textContainer: {
        paddingHorizontal: 5,
        borderRadius: 4,
        height: 22,
        flexDirection: 'row',
        flex: 1
    },
    leftTextContainer: {
        marginLeft: 14
    },
    rightTextContainer: {
        marginRight: 14
    },
    text: {
        fontSize: 14,
        lineHeight: 20,
        fontFamily: SceneUtils_1.default.whitney_semibold
    },
    button: {
        alignItems: "center",
        justifyContent: "center",
        marginHorizontal: 14
    },
    iconLogo: {
        resizeMode: "cover",
        width: 40,
        height: 40,
        borderRadius: 20
    },
    icon: {
        resizeMode: "contain",
        width: 20,
        height: 20
    },
    countContainer: {
        width: 24,
        height: 24,
        backgroundColor: SceneUtils_1.default.primary_color,
        justifyContent: 'center',
        alignItems: 'center',
        borderRadius: 12,
        position: 'absolute',
        right: 0
    },
    countText: {
        color: 'white',
        fontWeight: 'bold',
        fontSize: 12
    }
});
exports.default = FloatingActionItem;
//# sourceMappingURL=FloatingActionItem.js.map